package me.km.utils;

import java.util.ArrayList;
import java.util.List;
import me.km.exception.IllegalItemStackStringException;
import net.minecraft.block.Block;
import net.minecraft.entity.ai.attributes.AttributeModifier;
import net.minecraft.entity.ai.attributes.AttributeModifier.Operation;
import net.minecraft.item.Items;
import net.minecraft.inventory.EquipmentSlotType;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.JsonToNBT;
import net.minecraft.nbt.CompoundNBT;
import net.minecraft.nbt.ListNBT;
import net.minecraft.nbt.StringNBT;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

public class ItemStackUtils 
{
    // -----------------------------------------------------------------------------------
    // Attribute
    // -----------------------------------------------------------------------------------
    
    public enum Attribute
    {
        /**
         * value from 0 to 30;
         */
        ARMOR("generic.armor"),
        /**
         * value from 0 to 20;
         */
        ARMOR_TOUGHNESS("generic.armorToughness"),
        /**
         * value from 0 to 1.7E308;
         */
        ATTACK_DAMAGE("generic.attackDamage"),
        /**
         * value from 0 to 1;
         */
        KNOCKBACK_RESISTANCE("generic.knockbackResistance"),
        /**
         * value from 0 to 1.7E308;
         */
        MAX_HEALTH("generic.maxHealth"),
        /**
         * value from 0 to 1.7E308;
         */
        MOVEMENT_SPEED("generic.movementSpeed"),
        /**
         * value from 0 to 1024;
         */
        ATTACK_SPEED("generic.attackSpeed"),
        /**
         * value from -1024 to 1024;
         */
        LUCK("generic.luck");
        
        private final String name;
        
        Attribute(String name)
        {
            this.name = name;
        } 
        
        public String getName()
        {
            return name;
        }
    }
    
    public static void addAttribute(ItemStack stack, Attribute a, EquipmentSlotType slot, double amount, Operation op)
    {
        stack.addAttributeModifier(a.getName(), new AttributeModifier("modifier", amount, op), slot);
    }
    
    // -----------------------------------------------------------------------------------
    // Item-Flags
    // -----------------------------------------------------------------------------------
    
    public enum ItemFlag
    {
        /**
        * Setting to show/hide enchants
        */
        HIDE_ENCHANTS,
        /**
        * Setting to show/hide Attributes like Damage
        */
        HIDE_ATTRIBUTES,
        /**
        * Setting to show/hide the unbreakable State
        */
        HIDE_UNBREAKABLE,
        /**
        * Setting to show/hide what the ItemStack can break/destroy
        */
        HIDE_DESTROYS,
        /**
        * Setting to show/hide where this ItemStack can be build/placed on
        */
        HIDE_PLACED_ON,
        /**
        * Setting to show/hide potion effects on this ItemStack
        */
        HIDE_POTION_EFFECTS;
        
        public byte getBit()
        {
            return (byte) (1 << this.ordinal());
        }
    }

    public static void addItemFlag(ItemStack stack, ItemFlag... flags)
    {
        CompoundNBT com = stack.hasTag() ? stack.getTag() : new CompoundNBT();
        if(com == null) // this will never happen, but the compiler wants it
        {
            return;
        }
        int i = com.getInt("HideFlags");
        for (ItemFlag f : flags) 
        {
            i |= f.getBit(); // bitwise 'or'
        }
        com.putInt("HideFlags", i);
        stack.setTag(com);
    }
    
    public static void removeItemFlags(ItemStack stack, ItemFlag... flags) 
    {
        CompoundNBT com = stack.hasTag() ? stack.getTag() : new CompoundNBT();
        if(com == null) // this will never happen, but the compiler wants it
        {
            return;
        }
        int i = com.getInt("HideFlags");
        for (ItemFlag f : flags) 
        {
            i &= ~f.getBit(); // bitwise 'and' with inversed bits
        }
        com.putInt("HideFlags", i);
        stack.setTag(com);
    }
    
    public static boolean hasItemFlag(ItemStack stack, ItemFlag flag) 
    {
        if(!stack.hasTag())
        {
            return false;
        }
        CompoundNBT com = stack.getTag();
        if(com == null) // this will never happen, but the compiler wants it
        {
            return false;
        }
        byte b = flag.getBit();
        return (com.getInt("HideFlags") & b) == b;
    }
    
    // -----------------------------------------------------------------------------------
    // Converter
    // -----------------------------------------------------------------------------------
    
    public static String getNbtString(ItemStack stack)
    {
        return stack.write(new CompoundNBT()).toString();
    }
    
    public static ItemStack getStackFromNbtString(String s) throws IllegalItemStackStringException
    {
        try
        {
            CompoundNBT c = JsonToNBT.getTagFromJson(s);
            return ItemStack.read(c);
        }
        catch(Exception ex)
        {
            throw new IllegalItemStackStringException(ex.getMessage());
        }
    }
    
    // -----------------------------------------------------------------------------------
    // Misc
    // -----------------------------------------------------------------------------------

    public static void setLore(ItemStack stack, List<String> list) 
    {
        CompoundNBT com = stack.getOrCreateChildTag("display");
        ListNBT nbtList = new ListNBT();
        list.forEach(s -> nbtList.add(new StringNBT(s)));
        com.put("Lore", nbtList);
    }
    
    public static void addLore(ItemStack stack, String s) 
    {
        CompoundNBT com = stack.getOrCreateChildTag("display");
        ListNBT nbtList = com.getList("Lore", 8);
        nbtList.add(new StringNBT(s));
        com.put("Lore", nbtList);
    }
    
    public static List<String> getLore(ItemStack stack)
    {
        CompoundNBT com = stack.getOrCreateChildTag("display");
        ListNBT nbtList = com.getList("Lore", 8);
        ArrayList<String> list = new ArrayList<>();
        for(int i = 0; i < nbtList.size(); i++)
        {
            list.add(nbtList.getString(i));
        }
        return list;
    }    
    
    public static void addLore(ItemStack stack, String s, int i)
    {     
        CompoundNBT com = stack.getOrCreateChildTag("display");
        ListNBT list = com.getList("Lore", 8);                  
        if(i >= list.size())
        {
            list.add(new StringNBT(s));
        }
        else
        {
            list.set(i, new StringNBT(s));
        }           
        com.put("Lore", list);
    }
    
    public static void drop(World w, BlockPos pos, ItemStack stack)
    {
        Block.spawnAsEntity(w, pos, stack);
    }
    
    public static Item getItem(String stack) throws IllegalItemStackStringException
    {
        /*Item item = Item.getByNameOrId(stack);
        if(item == null)
        {
            Block b = Block.getBlockFromName(stack);
            if(b == null)
            {
                throw new IllegalItemStackStringException(stack);
            }
            return Item.getItemFromBlock(b);
        }
        return item;*/
        return Items.APPLE;
    }
}
